/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include <cstdio>

#include "classification.hpp"
#include "pixel_class.hpp"

void classify(histogram const& hist,
		centroid_set const& centers,
		classification* result)
{
	result->clear();
	for(histogram::const_iterator it=hist.begin();
			it!=hist.end(); ++it){
		if(*it){
			result->insert(std::make_pair(
					pixel_class(centers, it-hist.begin()),
					it-hist.begin()));
		}
	}
}


int class_dissimilarity(classification const& c12n,
		int c1, 
		int c2)
{
	typedef std::pair<classification::const_iterator,
		classification::const_iterator> pair;
	pair p1=c12n.equal_range(c1);
	pair p2=c12n.equal_range(c2);

	if(p1.first==p1.second || p2.first==p2.second) return 0;

	int min_dist=std::abs(p1.first->second-p2.first->second);

	for(classification::const_iterator it=p1.first; it!=p1.second;
			++it)
	for(classification::const_iterator jt=p2.first; jt!=p2.second;
			++jt){
		if(std::abs(jt->second-it->second)<min_dist)
			min_dist=std::abs(jt->second-it->second);
	}
	return min_dist;

}

int class_diameter(classification const& c12n, int c)
{
	typedef std::pair<classification::const_iterator,
		classification::const_iterator> pair;
	pair p1=c12n.equal_range(c);
	if(p1.first==p1.second) return 0;
	
	classification::const_iterator itt=p1.first;
	++itt;
	if(itt==p1.second) return 0;

	int max_dist=std::abs(p1.first->second-itt->second);

	for(classification::const_iterator it=p1.first;
			it!=p1.second; ++it){
		for(classification::const_iterator jt=it;
				jt!=p1.second; ++jt)
			if(std::abs(it->second-jt->second)>max_dist)
				max_dist=std::abs(it->second-jt->second);
	}
	return max_dist;
}
