/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef CLASSIFICATION_HPP
#define CLASSIFICATION_HPP

#include <map>
#include "histogram.hpp"
#include "centroid_set.hpp"

/// Holds the resulting pixel classification of an algorithm
///
/// The keys are the centroids indexes and the values are gray levels
/// that have been *classified* with that centroid.
typedef std::multimap<int, 
	histogram::gray_level_type> classification;

/// Classifies the gray levels in a histogram accroding
/// to the given centroid set and puts the result in
/// passed classification
void classify(histogram const& hist,
		centroid_set const& centers,
		classification* result);

/// Dissimilarity between two classes (given their indexes).
/// Defined as the distance between the two closest elements of
/// the class
int class_dissimilarity(classification const&,
		int c1, 
		int c2);

/// Diameter of a class. Defined as the distance between
/// the two farthest away elements
int class_diameter(classification const&, int c);

#endif // CLASSIFICATION_HPP
