/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include <algorithm>
#include <cstdio>

#include "compute_histogram.hpp"
#include "rgb_to_gray.hpp"

namespace 
{
	// Computes histogram for gray scale drawables
	void compute_histogram_gray(GimpDrawable* drawable,
			histogram* hist,
			int x, int y, int width, int height)
	{
		std::fill(hist->begin(), hist->end(), 0);

		pixel_region preg(drawable, x, y, width, height, false, false);

		for(pixel_region::iterator it=preg.begin();
				it!=preg.end(); ++it){
			pixel_region::chunk const& cnk=*it;
			for(guchar* ch=cnk.begin; ch!=cnk.end; ++ch)
				hist->operator[](*ch)++;
		}

	}

	// Computes histogram for RGB(A) drawables
	void compute_histogram_rgb(GimpDrawable* drawable,
			histogram* hist,
			int x, int y, int width, int height)
	{
		std::fill(hist->begin(), hist->end(), 0);

		pixel_region preg(drawable, x, y, width, height, false, false);

		for(pixel_region::iterator it=preg.begin();
				it!=preg.end(); ++it){
			pixel_region::chunk const& cnk=*it;
			for(guchar* ch=cnk.begin; ch!=cnk.end; ch+=drawable->bpp){
				guchar const level=rgb_to_gray(ch[0], ch[1], ch[2]);
				hist->operator[](level)++;
			
			}
		}
	}
} // anonymous namespace

/// Computes the histogram of a drawable
void compute_histogram(GimpDrawable* drawable, histogram* hist,
		int x, int y, int width, int height)
{
	switch(drawable->bpp){
		case 1:
			compute_histogram_gray(drawable, hist,
					x, y, width, height);
			break;
		case 3:
		case 4:
			compute_histogram_rgb(drawable, hist,
					x, y, width, height);
			break;

	}
}

