/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "dunn_fitness.hpp"


double dunn_fitness::operator() (histogram const& hist,
			centroid_set const& centers,
			classification const& c12n) const
{
	if(centers.size()<2) return 1;

	int min_distance=class_dissimilarity(c12n, 0, 1);
	for(unsigned i=0; i<centers.size(); ++i){
		for(unsigned j=i+1; j<centers.size(); ++j){
			int const d=
				class_dissimilarity(c12n, i, j);
			if(d<min_distance) min_distance=d;
		}
	}

	int max_diam=class_diameter(c12n, 0);
	for(unsigned i=0; i<centers.size(); ++i){
		int const diam=class_diameter(c12n, i);
		if(diam>max_diam)max_diam=diam;
	}
	if(max_diam==0) return 1;
	else return 10*double(min_distance)/double(max_diam);
}
