/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "dyn_pso_pass.hpp"
#include "dyn_pso.hpp"
#include "dyn_pso_utils.hpp"

double fitness_turi(std::vector<bool>& dyn_particle_position,std::vector<double>& centroids, std::vector<cluster>& hist);

void generate_centroids(int size, std::vector<double>& centroids,std::vector<cluster>& hist)
{
    std::vector<int> v; //Available grey levels
    int gl_number=0;
    int r;
    int cpt=0;
    centroids.clear();
    for (unsigned i=0; i<hist.size();++i)
    {
        if (hist[i].num_pixel==0)
            continue; //no pixels for this grey level
        v.push_back(i);
        gl_number++;
    }
    r=rand()%(gl_number);
    centroids.push_back((double)v[r]);
    cpt++;
    for(int i=0;i<size-1;++i)
    {
        if(gl_number> cpt)
        {
            do
            {
                r=rand()%(gl_number);

            }
            while(find(centroids.begin(), centroids.end(),(double) v[r])!=centroids.end());
            centroids.push_back((double)v[r]);
        }
        else //gl_number <= cpt
        {
            //generate a randomly values
            do
            {
                r=rand()%256;

            }
            while(find(centroids.begin(), centroids.end(),(double)r)!=centroids.end());
            centroids.push_back((double)r);
        }
        cpt++;
    }

}

param_info 
dyn_pso_pass::m_param_info[dyn_pso_pass::m_param_info_size]=
{
	param_info(1, 100, "Number of Iterations", "Number of passes to apply"),
	param_info(0.1, 1.0, "W", "Inertia"),
	param_info(0.1, 3.0, "C1", "First constant"),
	param_info(0.1, 3.0, "C2", "Second constant"),
	param_info(5, 100, "Swarm Size", "Swarm size"),
	param_info(5, 60, "Particle Size", "Particle size"),
	param_info(0.1, 1.0, "Seed", "Seed to initialize dyn_particles"),
	param_info(1, 10, "Velocity Range", "Velocity Range")	
};

void dyn_pso_pass::operator() (histogram const& hist,
			centroid_set& centroids,
			progress_indicator* ind) const
{	
			
			std::vector<double> cent(centroids.begin(),
			centroids.end());
			std::vector<cluster> histogram(256);
			for (unsigned i=0;i<256;++i)
			{
				histogram[i].num_pixel=hist[i];
			}
			int const nr_passes=this->pass::parameter_value(0).int_value();
			int const ss=this->pass::parameter_value(4).int_value();
			int const ps=this->pass::parameter_value(5).int_value();
			int const Velocity=this->pass::parameter_value(7).int_value();
			double const w=this->pass::parameter_value(1).double_value();
			double const c1=this->pass::parameter_value(2).double_value();
			double const c2=this->pass::parameter_value(3).double_value();
			double const Pini=this->pass::parameter_value(6).double_value();
			
			if (centroids.empty()) generate_centroids(ps,cent,histogram);
			dyn_pso_swarm sw(w,c1,c2,ss,ps,Pini,Velocity,cent,histogram,fitness_turi);
			for(int c=0;c<nr_passes;c++)
			{
				sw.update_swarm();
				ind->operator() (double(c)/nr_passes);
			}
			std::vector<bool>& bpp=sw.get_best_dyn_particle_position();
			std::cout<<"the best position: "<<sw.get_best_dyn_particle_cost()<<std::endl;
			centroids.clear();
			std::printf(" mask_size: %d centroid_size: %d \n",
				bpp.size(), cent.size());
				
			for (unsigned i=0;i<bpp.size();++i)
			{
				if (bpp[i]){
					std::printf("pushing %d\n", i);
					centroids.push_back(cent[i]);
				}
			}
			
			if(centroids.empty()){
				std::printf("EMPTY!!!!!!!!!\n");
				centroids.push_back(255);
				centroids.push_back(128);
				centroids.push_back(0);
			}
			centroids=cent;
			
			
}
;

