/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef FITNESS_FUNCTION_HPP
#define FITNESS_FUNCTION_HPP

#include "histogram.hpp"
#include "centroid_set.hpp"
#include "classification.hpp"

/// <Abstract> Used to evaluate the restults of a segmentation algorithm
class fitness_function
{
	public:
	/// Takes the display name of the fitness function
	fitness_function(char const* label): m_label(label) {}

	/// Main member take segmentation input and output and returns
	/// a eveluation value between 0.0 and 1.0
	virtual double operator() (histogram const& hist,
			centroid_set const& cent,
			classification const& cl) const=0;
	
	/// Name of the fitness_function
	char const* label() const { return m_label; }

	virtual ~fitness_function() {}
	
	private:
	char const * m_label;
};

#endif // FITNESS_FUNCTION_HPP
