/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef FITNESS_FUNCTION_REPERTOIRE_HPP
#define FITNESS_FUNCTION_REPERTOIRE_HPP

#include "fitness_function.hpp"

#include <map>
#include <string>

/// (Singleton) Holds a repertoire of fitness_function objects.
///
/// The fitness_function_repertoire is supposed to hold a
/// prototype for every possible kind of child class of pass.
class fitness_function_repertoire
{
	public:
		/// Access to unique instance
		static fitness_function_repertoire* instance()
		{
			if(!m_instance) m_instance=new fitness_function_repertoire();
			return m_instance;
		}

		/// Gets a prototype given a label (or NULL)
		fitness_function* find(char const* label);

		/// STL-lity
		typedef std::vector<fitness_function*>::const_iterator iterator;
		
		/// STL-lity
		iterator begin() const { return m_repertoire.begin(); }
		
		/// STL-lity
		iterator end() const { return m_repertoire.end(); }
	
		/// Register a new prototype in the repertoire
		void add(fitness_function* f) 
		{ m_repertoire.push_back(f); }

		/// Constructor of which will register a prototype
		struct registerer
		{
			registerer(fitness_function* p)
			{ 
				fitness_function_repertoire::instance()->add(p);
			}
		};

		/// STL-lity
		unsigned size() const { return m_repertoire.size(); }
	private:
		std::vector<fitness_function*> m_repertoire;
		static fitness_function_repertoire* m_instance;
		fitness_function_repertoire(){}
};

#endif // FITNESS_FUNCTION_REPERTOIRE_HPP
