/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef HISTOGRAM_HPP
#define HISTOGRAM_HPP

#include <cstdlib> // for std::size_t

/// Gives the number of pixels of a given gray level for each possible
/// gray level.
/// (This is mainly a storage facility)
struct histogram
{
	/// Number of possible gray levels
	enum { gray_scale_size=256 };
	
	/// Stores a gray level (a much as gray_scale_size values) 
	/// (not used herin but provided for outside world)
	typedef unsigned char gray_level_type;

	/// Stores the number of pixels of a given gray level
	typedef int amount_type;

	/// STL-lity
	typedef amount_type* iterator;
	
	/// STL-lity
	typedef amount_type const* const_iterator;

	/// Holds the number of pixels for each of the 256 possible
	/// gray levels
	amount_type data[gray_scale_size];

	/// Handy access
	amount_type& operator[] (std::size_t t) { return data[t]; }
	
	/// Handy access
	amount_type operator[]  (std::size_t t) const { return data[t]; }

	/// STL-like access
	iterator begin() { return data; }
	iterator end() { return data+gray_scale_size; }

	/// STL-like access
	const_iterator begin() const { return data; }
	const_iterator end() const { return data+gray_scale_size; }

	/// Convenience facility
	std::size_t size() const { return gray_scale_size; }
};

#endif // HISTOGRAM
