/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef JACCARD_DISTANCE_HPP
#define JACCARD_DISTANCE_HPP

#include <libgimp/gimp.h>
#include "classification.hpp"
#include "centroid_set.hpp"

/// Computes Jaccard distance between two segmentation results
///
/// image: The original (unmodified) image on which a segmentation was applied
///          (RGB or gray scale)
/// image_c12n: Results of the pixel classification applied on image
/// ground_truth: The manual segmentation corresponding to image (RGB or
///	gray scale)
/// ground_hist: Histogram of (x,y,w,h) region of ground_truth
/// x,y,w,h: The area on which to restrict comparison
///
/// The output consists of one std::pair per possible gray level in the
/// (x,y,h,w) region of ground_truth (in order). Each pair contains the center
/// index of image_c12n that corresponds the most to that class in ground_truth,
/// plus the jaccard index estimation of the correspondence. 
void jaccard_distance(GimpDrawable* image, 
	classification const& image_c12n,
	centroid_set const& image_centers,
	GimpDrawable* ground_truth,
	histogram const& ground_hist,
	int x, int y, int w, int h,
	std::vector<std::pair<int, double> >* out
	);

#endif // JACCARD_DISTANCE_HPP

