/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "kmeans_pass.hpp"
#include "pixel_class.hpp"
#include "classification.hpp"

#include <cstdio>

//pass_repertoire::registerer kmeans_pass::m_reg(new kmeans_pass());

param_info 
kmeans_pass::m_param_info[kmeans_pass::m_param_info_size]=
{
	param_info(1, 100, "Number Of Passes", "Number of iterations to perform"),
	param_info(2, 100, "Number Of Classes", "Intial number of classes"),
};

void kmeans_pass::print() const
{
	std::printf("%s %d \n",
			label(),
			parameter_value(0).int_value());
}

void kmeans_pass::operator() (histogram const& hist,
		centroid_set& centroids,
		progress_indicator* progress) const
{	
	int const nr_passes=this->pass::parameter_value(0).int_value();
	int const nr_classes=this->pass::parameter_value(1).int_value();
	int pass_count=nr_passes;
	
	if(centroids.empty()){
		// Initialize the centroids to the three most
		// common gray levels in the images
		std::map<int, histogram::gray_level_type> mp;
		for(histogram::const_iterator it=hist.begin(); it!=hist.end(); ++it){
			if(*it)
				mp.insert(std::make_pair(*it, it-hist.begin()));
		}
		
		std::map<int, histogram::gray_level_type>::iterator it=mp.begin();
		for(int i=0; i<nr_classes; ++i){
			if(it==mp.end()) break;
			centroids.push_back(it->second);
			++it;
		}
	}

	classification c12n;

	while(pass_count--){
		// First go and assign a class index to each gray level
		classify(hist, centroids, &c12n);
		
		// Adjust each centroid in turn
		for(centroid_set::iterator it=centroids.begin();
				it!=centroids.end(); ++it){
			
			std::pair<classification::iterator,
				classification::iterator> pr=c12n.equal_range(
					it-centroids.begin());

			if(pr.first==pr.second) continue;

			// Sum of (nr_pixels*gray_level + nr_pixels*gray_level...)
			int partial_sum=0;
			// Sum of (nr_pixels + nr_pixels + ...
			int coef_sum=0; 

			// Compute partial_sum&coef_sum
			for(classification::iterator jt=pr.first; 
					jt!=pr.second; ++jt){
				coef_sum+=hist[jt->second];
				partial_sum+=hist[jt->second]*jt->second;
			}
			*it=double(partial_sum)/coef_sum;
		}
		progress->operator() (double(nr_passes-pass_count)/nr_passes);
	}// while (pass_count)
}


