/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "liu_yang_fitness.hpp"
#include<cmath>
#include<cstdlib>

double abs(double d){ return d>0?d:-d; }

double liu_yang_fitness::operator() (histogram const& hist,
			centroid_set const& centers,
			classification const& c12n) const
{
	double result=0;
	double intra_distance=0;
	int num_pixel_image=0;
	int num_pixel_class=0;
	
	if(centers.size()<2) return 1;
	
	for(centroid_set::const_iterator it=centers.begin(); it!=centers.end(); ++it)
	{
		std::pair<classification::const_iterator,
			classification::const_iterator> pr=c12n.equal_range(it-centers.begin());
			
		for(classification::const_iterator jt=pr.first; jt!=pr.second;++jt)
		{
			intra_distance+=::abs(*it-jt->second);
			num_pixel_class+=hist[jt->second];
			
		}
		result+=(intra_distance*intra_distance)/(double)sqrt(double(num_pixel_class));
		num_pixel_image+=num_pixel_class;
		intra_distance=0;
		num_pixel_class=0;
	}
	result=(1.0/(1000+num_pixel_image))*sqrt(centers.size())*result;
	
	return 1/result;
}

	
