/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include <string>
#include <cstdio>
#include <algorithm>

#include "load_manual_segmentation.hpp"


GimpDrawable* load_manual_segmentation(gint32 image_id)
{
	gchar* const cfilename=gimp_image_get_filename(image_id);
	if(!cfilename) return 0;

	std::string const filename=cfilename;
	std::string extension;
	std::string basename;

	// Try and extract extension and base name
	{
		std::string::const_reverse_iterator it=std::find(filename.rbegin(),
				filename.rend(), '.');
		if(it!=filename.rend()){
			extension.assign(it.base(), filename.end());
			basename.assign(filename.begin(), 
					it.base()-1);
		} else {
			extension.clear(); // already empty, but anyway...
			basename=filename;
		}

		//std::cout<<"Filename:"<<filename<<" Basename:"<<basename<<" Ext:"<<extension<<std::endl;
	}

	std::string new_filename=basename+"_seg."+extension;

	// try to open the file first
	FILE* fp=fopen(new_filename.c_str(), "r");
	if(!fp) return 0;
	else fclose(fp);


	gint32 ground_truth_image_id=gimp_file_load(GIMP_RUN_NONINTERACTIVE,
			new_filename.c_str(), new_filename.c_str());
	if(ground_truth_image_id<0) return 0;

	if(gimp_image_base_type(ground_truth_image_id)!=GIMP_GRAY)
		if(!gimp_image_convert_grayscale(ground_truth_image_id)) {
			gimp_image_delete(ground_truth_image_id);
			return 0;
		}

	gint num_layers;
	gint *layers;
	layers=gimp_image_get_layers(ground_truth_image_id, &num_layers);
	if(num_layers<=0) {
		gimp_image_delete(ground_truth_image_id);
	}


	GimpDrawable* truth=gimp_drawable_get(layers[0]);
	gint drawable_id=gimp_image_get_active_drawable(image_id);
	if(truth->width!=unsigned(gimp_drawable_width(drawable_id)) ||
			truth->height!=unsigned(gimp_drawable_height(drawable_id))){
		gimp_drawable_detach(truth);
		gimp_image_delete(ground_truth_image_id);
		return 0;
	}
	return truth;
}
