/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef MULTIPASS_BATCH_DIALOG_HPP
#define MULTIPASS_BATCH_DIALOG_HPP

#include <string>
#include "pass.hpp"

/// Creates a gimp dialog to allow the edition
/// of the passed collection of passes and their parameters,
/// the edition/loading/saving of a list of images files ("benchmarks"),
/// and the specification of a filename where segmentation results will
/// be saved.
///
/// Returned value tells if the user confirmed(true) or canceled(false)
/// The passed argument is modified in place
///
/// The pointers to passes are newed and deleted using C++ standard 
/// operators
///
/// \warning Caller is responsible of invoking gimp_ui_init()
/// \warning The current available repertoire of passes is fetched
/// from pass_repertoire (make sure to register your passes with it)
bool multipass_batch_dialog(std::vector<pass*>& algo,
		std::vector<std::string>& benchmark,
		std::string& results);

#endif // MULTIPASS_BATCH_DIALOG_HPP
