/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef PASS_REPERTOIRE_HPP
#define PASS_REPERTOIRE_HPP

#include "pass.hpp"

#include <map>
#include <string>

/// (Singleton) Holds a repertoire of prototypes of passes.
///
/// The pass_repertoire is supposed to hold a prototype for every possible
/// kind of child class of pass.
///
class pass_repertoire
{
	public:
		/// Access to unique instance
		static pass_repertoire* instance()
		{
			if(!m_instance) m_instance=new pass_repertoire();
			return m_instance;
		}

		/// Gets a prototype given a label (or NULL)
		pass* find(char const* label);

		/// STL-lity
		typedef std::vector<pass*>::const_iterator iterator;
		
		/// STL-lity
		iterator begin() const { return m_repertoire.begin(); }
		
		/// STL-lity
		iterator end() const { return m_repertoire.end(); }
	
		/// Register a new prototype in the repertoire
		void add(pass* p) 
		{ m_repertoire.push_back(p); }

		/// Constructor of which will register a prototype
		struct registerer
		{
			registerer(pass* p)
			{ 
				std::printf("Adding to rep:\n");
				p->print();

				pass_repertoire::instance()->add(p);
			}
		};

	private:
		std::vector<pass*> m_repertoire;
		static pass_repertoire* m_instance;
		pass_repertoire(){}
};

#endif // PASS_REPERTOIRE_HPP
