/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef PIXEL_CLASSIFIER_HPP
#define PIXEL_CLASSIFIER_HPP

#include <libgimp/gimp.h>

#include "histogram.hpp"
#include "classification.hpp"
#include "centroid_set.hpp"

/// <Abstract> Gives the effective class of each pixel on a
/// GimpDrawable. Should be called after pixel classification on the
/// histogram has been performed on the drawable to fine tune results
/// on a per pixel basis.
class pixel_classifier
{
	public:
	/// Takes display name label
	pixel_classifier(char const* label): m_label(label) {}

	/// Returns the class number of given pixel
	/// rgn is initialized to cover the whole drawable,
	/// (x, y) is the pixel to classify. 
	/// geom_pos_factor (between 0 and 1) tells how important
	/// the position of the pixel is for its classification
	virtual centroid_set::size_type operator() (
			histogram const& hist,
			centroid_set const& centers,
			classification const& cls,
			GimpDrawable* drawable,
			GimpPixelRgn* rgn,
			guchar level,
			gint x, gint y,
			double geom_pos_factor) const=0;
	
	/// Access to display label 
	char const* label() const { return m_label; }

	/// Virtual dtor recommended for ABSs
	virtual ~pixel_classifier(){}

	private:
	char const* m_label;
};


#endif // PIXEL_CLASSIFIER_HPP
