/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef PIXEL_CLASSIFIER_REPERTOIRE_HPP
#define PIXEL_CLASSIFIER_REPERTOIRE_HPP

#include "pixel_classifier.hpp"

#include <map>
#include <string>

/// (Singleton) Holds a repertoire of pixel_classifier objects.
///
/// The pixel_classifier_repertoire is supposed to hold a
/// prototype for every possible kind of child class of pixel_classifier.
class pixel_classifier_repertoire
{
	public:
		/// Access to unique instance
		static pixel_classifier_repertoire* instance()
		{
			if(!m_instance) m_instance=new pixel_classifier_repertoire();
			return m_instance;
		}

		/// Gets a prototype given a label (or NULL)
		pixel_classifier* find(char const* label);

		/// STL-lity
		typedef std::vector<pixel_classifier*>::const_iterator iterator;
		
		/// STL-lity
		iterator begin() const { return m_repertoire.begin(); }
		
		/// STL-lity
		iterator end() const { return m_repertoire.end(); }
	
		/// Register a new prototype in the repertoire
		void add(pixel_classifier* f) 
		{ m_repertoire.push_back(f); }

		/// Constructor of which will register a prototype
		struct registerer
		{
			registerer(pixel_classifier* p)
			{ 
				pixel_classifier_repertoire::instance()->add(p);
			}
		};

		/// STL-lity
		unsigned size() const { return m_repertoire.size(); }
	private:
		std::vector<pixel_classifier*> m_repertoire;
		static pixel_classifier_repertoire* m_instance;
		pixel_classifier_repertoire(){}
};

#endif // PIXEL_CLASSIFIER_REPERTOIRE_HPP
