/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include <vector>
#include <map>
#include <algorithm>
#include <cmath>


#include "pos_pixel_classifier.hpp"
#include "rgb_to_gray.hpp"
#include "pixel_class.hpp"
#include <cstdio>

// fills res with classes of the neighbourhood pixels (and pixel itself)
// of (x,y) ( key the class number, value the number of times it 
// appears
void neighbourhood(GimpPixelRgn* rgn, int x, int y,
		centroid_set const& centers,
		std::map<int, int>* res)
{
	struct xy { guint x, y;};
	xy pos []={
		{x,y},
		{ x+1, y},
		{ x, y+1},
		{ x+1, y+1},
		{ x-1, y},
		{ x, y-1},
		{x-1, y-1},
		{x-1, y+1},
		{x+1, y-1}};
	guchar pix[4];
	res->clear();
	for(xy* p=pos; p<pos+sizeof(pos)/sizeof(*pos); ++p){
		if(p->x < rgn->x || p->x > rgn->x+rgn->w-1 ||
				p->y< rgn->y || 
				p->y > rgn->y+rgn->h-1) continue;
		gimp_pixel_rgn_get_pixel(rgn, pix, p->x, p->y);
		guchar level;
		if(rgn->bpp==1){
			level=pix[0];
		} else {
			level=rgb_to_gray(pix[0], pix[1], pix[2]);
		}

		res->operator[](pixel_class(centers, level))+=1;
	}
}

centroid_set::size_type pos_pixel_classifier::operator() (
		histogram const& hist,
		centroid_set const& centers,
		classification const& c12n,
		GimpDrawable* drawable,
		GimpPixelRgn* rgn,
		guchar level,
		gint x, gint y,
		double geom_pos_factor) const
{	
	std::map<int, int> neigh_classes;
	neighbourhood(rgn, x, y, centers, &neigh_classes);
	
	// Key: distance, Value: class number
	std::map<double, int> class_distance;
	for(std::map<int, int>::iterator it=neigh_classes.begin();
			it!=neigh_classes.end(); ++it){
		double dist=
			std::abs(double(level)-centers[it->first])/
			double(it->second);
		class_distance.insert(std::make_pair(dist, it->first));
	}

	return  class_distance.begin()->second;
}
