/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <map>

#include "pso_pass.hpp"
#include "swarm_def.hpp"

param_info 
pso_pass::m_param_info[pso_pass::m_param_info_size]=
{
	param_info(1, 100, "Number of Iterations", "Number of passes to apply"),
	param_info(0.1, 1.0, "W", "Inertia"),
	param_info(0.1, 3.0, "C1", "First constant"),
	param_info(0.1, 3.0, "C2", "Second constant"),
	param_info(5, 100, "Swarm Size", "Swarm size"),
	param_info(5, 60, "Particle Size", "Partical size"),
	param_info(1, 10, "Velocity Range", "Velocity Range")	
};

void pso_pass::operator() (histogram const& hist,
			centroid_set& centroids,
			progress_indicator* ind) const
{	
			
			int nr_passes=this->pass::parameter_value(0).int_value();
			int ss=this->pass::parameter_value(4).int_value();
			int ps=this->pass::parameter_value(5).int_value();
			int velocity=this->pass::parameter_value(6).int_value();
			double w=this->pass::parameter_value(1).double_value();
			double c1=this->pass::parameter_value(2).double_value();
			double c2=this->pass::parameter_value(3).double_value();
			
			if (centroids.empty())
			{
				//fill them
				std::map<int, histogram::gray_level_type> mp;
				for(histogram::const_iterator it=hist.begin(); it!=hist.end(); ++it)
				{
					if(*it) mp.insert(std::make_pair(*it, it-hist.begin()));
				}

				std::map<int, histogram::gray_level_type>::iterator it=mp.begin();
				for(int i=0; i<ps; ++i)
				{
					if(it==mp.end())
						break;
					centroids.push_back(it->second);
					++it;
				}
			
                       
			}
			
			swarm sw(w,c1,c2,ss,ps,velocity,centroids,hist);
			
			for(int c=0;c<nr_passes;c++)
			{
				sw.update_swarm();
				ind->operator() (double(c)/nr_passes);
			}			
			centroids=sw.get_best_particle_position();
}
;

