/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef TEMPLATE_PASS_HPP
#define TEMPLATE_PASS_HPP

#include "pass.hpp"

/// A template for implementing a new pass
class template_pass: public pass
{
	private:
		/// The number of parameters we need
		enum { m_param_info_size=2 };

		/// Holds a param_info for each paramter we take
		static param_info m_param_info[m_param_info_size];

	public:
	/// Pass a label, an array of param_info and the size of the
	/// array to parent class
	template_pass():
		pass("Template", m_param_info, m_param_info_size)
	{}

	/// The actual implementation of the algorithm
	/// historgram: Holds the number of pixel of each possible
	///    gray level in the image.
	/// centers: The input class centers. The algorithm should update
	///   them/improve them.
	/// ind: Can be used to report the progression
	virtual void operator() (histogram const&,
			centroid_set& centers,
			progress_indicator* ind) const;

	/// Must return a new copy of *this
	virtual pass* clone() const { return new template_pass(*this); }
};
#endif // TEMPLATE_PASS_HPP 
