#include <cassert>
#include "pass_repertoire.hpp"

param_info params_info[]={
	param_info(0, 100, "integer", "An integer parameter"),
	param_info(0.0, 100.0, "double", "A double parameter")
};
std::size_t const nr_params=sizeof(params_info)/sizeof(*params_info);

param_info params_info2[]={
	param_info(0, 200, "Sigma_I", "An integer parameter2"),
	param_info(0.0, 200.0, "Alpha_2", "A double parameter2"),
	param_info(0, 2000, "Number of passes", "The number of times to")
};
std::size_t const nr_params2=sizeof(params_info2)/sizeof(*params_info);

class pass1: public pass 
{
	public:
	pass1(): pass("pass1", params_info, nr_params){}
	virtual void operator() (histogram const&,
			centroid_set&,
			progress_indicator*) const{}
	virtual pass* clone() const{ return new pass1(*this); }
};
pass_repertoire::registerer r(new pass1());

class pass2: public pass 
{
	public:
	pass2(): pass("pass2", params_info2, nr_params2){}
	virtual void operator() (histogram const&,
			centroid_set&,
			progress_indicator*) const{}
	virtual pass* clone() const{ return new pass2(*this); }
};
pass_repertoire::registerer u(new pass2());

int main()
{
	pass_repertoire& pr=*pass_repertoire::instance();

	int i=0;
	for(pass_repertoire::iterator it=pr.begin();
			it!=pr.end(); ++it){
		pass* p=(*it)->clone();
		for(unsigned j=0; j<p->nr_parameters(); ++j){
			assert(p->parameter_value(j)==(*it)->parameter_value(j));
			assert(p->parameter_value(j).type()==
					(*it)->parameter_value(j).type());
		}
		assert(std::strcmp(p->label(), (*it)->label())==0);
		delete p;
		++i;
	}
	assert(i>=2);
}
