#include <cstdio>
#include <cassert>

#include "pass.hpp"
#include "ui_build.hpp"

param_info params_info[]={
	param_info(0, 100, "integer", "An integer parameter"),
	param_info(0.0, 100.0, "double", "A double parameter")
};
std::size_t const nr_params=sizeof(params_info)/sizeof(*params_info);

param_info params_info2[]={
	param_info(0, 200, "Sigma_I", "An integer parameter2"),
	param_info(0.0, 200.0, "Alpha_2", "A double parameter2"),
	param_info(0, 2000, "Number of passes", "The number of times to")
};
std::size_t const nr_params2=sizeof(params_info2)/sizeof(*params_info);

class pass1: public pass 
{
	public:
	pass1(): pass("pass1", params_info, nr_params){}
	virtual void operator() (histogram const&,
			centroid_set&,
			progress_indicator*) const {}
	virtual pass* clone() const{ return new pass1(*this); }
};

class pass2: public pass 
{
	public:
	pass2(): pass("pass2", params_info2, nr_params2){}
	virtual void operator() (histogram const&,
			centroid_set&,
			progress_indicator*) const {}
	virtual pass* clone() const{ return new pass2(*this); }
};

// Prints a param value/info
void print(param_info const& pi, param const& p)
{
	std::printf("%s: ", pi.label());
	switch(p.type()){
		case param::type_info::int_type:
			std::printf("%d", p.int_value());
			break;
		case param::type_info::double_type:
			std::printf("%f", p.double_value());
			break;
		default:
			break;
	}
	std::printf("\n");
}

// Callback that finishs the app
void quit_callback(GtkWidget* widget, gpointer data)
{ gtk_main_quit(); }

int main(int argc, char* argv[])
{
	gtk_init(&argc, &argv);

	GtkWidget* main_window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(main_window), "ui_build_test");
	g_signal_connect(G_OBJECT(main_window),
			"destroy",
			G_CALLBACK(quit_callback),
			NULL);

	GtkWidget* vbox=gtk_vbox_new(false, 10);
	gtk_container_add(GTK_CONTAINER(main_window),
			vbox);
	pass1 p1;
	p1.parameter_value(0).int_value()=120;
	pass* p1_clone=p1.clone();
	assert(p1_clone->parameter_value(0).int_value()==120);

	gtk_box_pack_start(GTK_BOX(vbox), 
			ui_build(&p1),
			false,
			false,
			0);
	pass2 p2;
	gtk_box_pack_start(GTK_BOX(vbox), 
			ui_build(&p2),
			false,
			false,
			0);
	gtk_widget_show(vbox);
	gtk_widget_show(main_window);
	gtk_main();

	for(unsigned i=0; i<p1.nr_parameters(); ++i)
		print(p1.parameter_info(i), p1.parameter_value(i));
	for(unsigned i=0; i<p2.nr_parameters(); ++i)
		print(p2.parameter_info(i), p2.parameter_value(i));
	return 0;
}

