/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "turi_fitness.hpp"
#include "dyn_pso_utils.hpp"
#include <cmath>
#include <cstdlib>

double fitness_turi(std::vector<bool>& particle_position,std::vector<double>& centroids, std::vector<cluster>& hist)
{
    //Initialise the histogram according to particle_position
    double inter=255, intra=0; //for the fitness function
    int np=0;
    double V; //the fitness function value
    double normal=0;//normal distribution
    double nearest;
    double dist;
    for (unsigned j=0;j<hist.size();++j) //for each object
    {
        nearest=255;
        if(hist[j].num_pixel==0)
        {
            hist[j].num_class=256;
            continue;
        }//No object(pixel) of this value
        for (unsigned int k=0;k<particle_position.size();++k)
        {
            if (particle_position[k]==0)
            {
                continue;
            }//X[i]==0
            dist=std::abs(j-centroids[k]);
            if(dist<nearest)
            {
                nearest=dist;
                hist[j].num_class=k;

            }
        }
    }

    //calculating the fitness fuction according to Turi
    for (unsigned j=0; j<hist.size();++j)
    {
        if(hist[j].num_pixel==0)
            continue;//No object(pixel) of this value
        intra=intra+hist[j].num_pixel*std::abs(j-centroids[hist[j].num_class]);
        np=np+hist[j].num_pixel;
    }
    intra=intra/np;
    for (unsigned j=0;j<centroids.size();++j)
    {
        if (particle_position[j]==0)
            continue;
        for(unsigned int k=j+1;k<centroids.size();++k)
        {
            if (particle_position[k]==0)
                continue;
            if(std::abs(centroids[j]-centroids[k])<inter)
                inter=std::abs(centroids[j]-centroids[k]);
        }
    }
    for (int j=0;j<12;++j)
    {
        normal=normal+myrandom(0)();
    }
    normal=normal-6; //generate N(0,1)
    V=(1*(/*normal+*/2)+3)*(intra/inter);//Turi clustering validity technique
    return V;

}

double turi_fitness::operator() (histogram const& hist,
		centroid_set const& centers,
		classification const& c12n) const
{
	std::vector<bool> ones(centers.size(), true);
	std::vector<double> cent(centers);

	std::vector<cluster> clusters(256);
	for(classification::const_iterator it=c12n.begin();
			it!=c12n.end(); ++it){
		clusters[it->second].num_pixel=hist[it->second];
		clusters[it->second].num_class=it->first;
	}
	double const ret=fitness_turi(ones, cent, clusters) ;
	return 1/ret;
}

