/// Multipass: A gimp plug-in for image segmentation
///
/// Copyright (C) 2007  Anis Benyelloul (anis.benyelloul@gmail.com) and
/// 			Said-Eddine Bensalem (said.bensalem@gmail.com)
///
/// This program is free software: you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation, either version 3 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include <cassert>

#include "ui_build.hpp"

/// Callback used by create_input_widget
void range_value_changed_handler(GtkWidget* range,
		gpointer user_data)
{
	param* p=static_cast<param*>(user_data);
	switch(p->type()){
		case param::type_info::int_type:
			p->int_value()=int(
				gtk_range_get_value(GTK_RANGE(range)));
			break;
		case param::type_info::double_type:
			p->double_value()=
				gtk_range_get_value(GTK_RANGE(range));
			break;
	}
}

/// Callback used by create_input_widget
void spin_button_value_changed_handler(GtkWidget* spin,
		gpointer user_data)
{
	param* p=static_cast<param*>(user_data);
	switch(p->type()){
		case param::type_info::int_type:
			p->int_value()=int(
				gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin)));
			break;
		case param::type_info::double_type:
			p->double_value()=
				gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin));
			break;
	}
}

// (Helper function)
// Constructs and returns an input widget that will allow the 
// entry of a value of a parameter as specified by param_info and
// whose signals will be setup to update the passed param
GtkWidget* create_input_widget(param_info const& pi,
		param* p)
{
	GtkWidget* input;
	switch(pi.type()){
		case param::type_info::int_type:
		{
			input=gtk_hscale_new_with_range(
						pi.int_start(),
						pi.int_end(),
						1.0);
			gtk_scale_set_value_pos(GTK_SCALE(input),
					GTK_POS_LEFT);
			g_signal_connect(G_OBJECT(input),
					"value-changed",
					G_CALLBACK(range_value_changed_handler),
					p);

			gtk_range_set_value(GTK_RANGE(input),
					p->int_value());
			break;
		} // case param::type_info::int_type
		case param::type_info::double_type:
		{
			input=gtk_spin_button_new_with_range(
					pi.double_start(),
					pi.double_end(),
					0.1);
			gtk_spin_button_set_digits(GTK_SPIN_BUTTON(input),
					3);
			g_signal_connect(G_OBJECT(input),
					"value-changed",
					G_CALLBACK(spin_button_value_changed_handler),
					p);
			
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(input),
					p->double_value());
			break;
		} // case param::type_info::double_type
		default:
			assert(false); // should never happen
	}// switch
	return input;
}

GtkWidget* ui_build(pass* p, void (*callback)(GtkWidget*, gpointer))
{
	GtkWidget* vbox=gtk_vbox_new(false, 0);

	for(unsigned i=0; i<p->nr_parameters(); ++i){
		GtkWidget* hbox=gtk_hbox_new(false, 10);
		GtkWidget* label=gtk_label_new(p->parameter_info(i).
				label());
		GtkWidget* input=create_input_widget(p->parameter_info(i),
				&p->parameter_value(i));
		if(callback){
			g_signal_connect(G_OBJECT(input),
					"value-changed",
					G_CALLBACK(callback),
					0);
		}

		gtk_box_pack_start(GTK_BOX(hbox),
				label,
				false, false, 0);
		gtk_box_pack_start(GTK_BOX(hbox),
				input,
				true, true, 0);
		gtk_widget_show(input);
		gtk_widget_show(label);
		gtk_widget_show(hbox);

		gtk_box_pack_start(GTK_BOX(vbox),
				hbox,
				false, false, 0);
	}
	gtk_widget_show(vbox);
	return vbox;
}

